package DSL::HTML::Text;
use strict;
use warnings;

use Carp qw/croak/;
croak "deprecated";

1;

__END__

=head1 NAME

DSL::HTML::Text - Used internally by L<DSL::HTML>

=head1 DEPRECATED

This module is deprecated.

=head1 AUTHORS

Chad Granum L<exodist7@gmail.com>

=head1 COPYRIGHT

Copyright (C) 2013 Chad Granum

DSL-HTML is free software; Standard perl license (GPL and Artistic).

DSL-HTML is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE. See the license for more details.
